#include "integrate.h"

#define			MAXECDPTS		16
#define 		CHROMSUB		MAXECDPTS*30	// 16 hz * 30 seconds

float ichrom[CHROMSUB][INTCHS];			// chrom array

char *intfile = "intres.txt";
float prvCalH[INTCHS];					// prev cals height
int intinc = 0;

typedef struct {
	float 	max;
	int 	maxpt;
	float 	min;
	int 	minpt;
	int 	lstpt;
} INTresStruct;

INTresStruct res;

// returns the max and min via res array between p1 and p2
float stats( int ch, int p1, int p2, int lstpt ) {
	int i;
	float max, min;
	int maxpt, minpt;

	max = ichrom[0][ch];
	maxpt = 0;
	min = ichrom[0][ch];
	minpt = 0;
	
	for (i=p1; i<p2; i++) {
		if ( ichrom[i][ch] > max ) {
			max = ichrom[i][ch];
			maxpt = i;
		}
		if ( ichrom[i][ch] < min ) {
			min = ichrom[i][ch];
			minpt = i;
		}
	}

	res.max = max;
	res.maxpt = maxpt;
	res.min = min;
	res.minpt = minpt;

	return max;
}

float subtractbaseline( int ch ) {

	int i, lstpt;
	float top, pt1x, pt2x;
	float pt1y, pt2y;
	float m, b;

	// find last data point in ichrom array
	for(i=0; i<CHROMSUB; i++ ) {
		if ( ichrom[i][ch] == -1 ) {
			lstpt = i-1;
			break;	
		}
	}
	res.lstpt = lstpt;
	
	stats( ch, 0, lstpt, lstpt );
	top = res.maxpt;

	// left side of peak
	stats( ch, 0, top, lstpt );
	pt1x = res.minpt;
	pt1y = res.min;

	// right side of peak
	stats( ch, top, lstpt, lstpt );
	pt2x = res.minpt;
	pt2y = res.min;

	m = ( pt2y - pt1y ) / ( pt2x - pt1x );
	b = pt1y - m * pt1x;

	// subtract baseline
	for( i=0; i<lstpt; i++ )
		ichrom[i][ch] -= m * i + b;
	
	stats( ch, 0, lstpt, lstpt );
	
	return res.max;

}

// returns the peak height
float hght( int ch ) {

	subtractbaseline( ch );
	subtractbaseline( ch );

	return subtractbaseline( ch );
	
}

void resetintfile( ) {
	FILE *int_file = NULL;

	if ((int_file = fopen( intfile, "w" )) == NULL)
		printf ("=====Cannot open %s\n", intfile);

	fprintf( int_file, "SSV, H, conc\n" );

	fclose( int_file );

}

// calculates the mixing ratio of current injection
float integrate ( int ch, int ST, int SP, int t, long resp, short ssv, short calssv, float calval ) {
	FILE *int_file = NULL;
	float H = 0.0;
	float MR = 0.0;
	int i;

	// in range
	if ( t == ST-1 ) {
		for ( i=0; i<CHROMSUB; i++ ) { ichrom[i][ch] = -1; }
		intinc = 0;
		return -1;
	}
	else if (( t >= ST ) && ( t <= SP )) {
		if ( resp > -1 )
			ichrom[intinc++][ch] = resp * 1.0;
		return -2;
	}
	else if (( t == SP+1 ) && ( ichrom[0][ch] != -1 )) { 
		if ((int_file = fopen( intfile, "a+" )) == NULL)
			printf ("=====Cannot open %s\n", intfile);

		H = hght( ch );

		if ( ssv != calssv ) {
			MR = (H / prvCalH[ch]) * calval;
			prvMR[ch] = MR;
		}
		else if ( ssv == calssv )
			prvCalH[ch] = H;

		fprintf( int_file, "%3u, %10.2f, %7.2f\n", ssv, H, prvMR[ch] );

		fclose( int_file );

		for ( i=0; i<CHROMSUB; i++ ) { ichrom[i][ch] = -1; }

		return prvMR[ch];
	}

	return -1;

}

float MRres( int ch ) {

	return prvMR[ch];

}